package net.reslate.islandsurvivalcraft.interaction.items;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map.Entry;

import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.Event.Result;
import org.bukkit.event.block.Action;
import org.bukkit.event.entity.EntityPickupItemEvent;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.event.inventory.InventoryType.SlotType;
import org.bukkit.event.player.PlayerDropItemEvent;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.event.player.PlayerItemHeldEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.Recipe;
import org.bukkit.inventory.meta.ItemMeta;

import net.reslate.islandsurvivalcraft.IslandSurvivalCraft;

public class ItemVariantManager implements Listener {
    private final IslandSurvivalCraft islandSurvivalCraft;

    public ItemVariantManager(IslandSurvivalCraft islandSurvivalCraft) {
        this.islandSurvivalCraft = islandSurvivalCraft;
    }

    public void loadAllItems() {
        RegisteredItemVariants[] items = RegisteredItemVariants.values();
        for (int i = 0; i < items.length; i++) {
            if (items[i].getVariedItem().needInitialization()) {
                items[i].getVariedItem().initialize(islandSurvivalCraft);
            }
            Recipe currentRecipe = items[i].getVariedItem().getRecipe();
            if (currentRecipe != null) {
                islandSurvivalCraft.getServer().addRecipe(currentRecipe);
            }
        }

        Collection<? extends Player> players = islandSurvivalCraft.getServer().getOnlinePlayers();
        for (Player player : players) {
            initializeInventoryVariedItems(player.getInventory());
        }
    }

    public void unloadAllItems() {
        RegisteredItemVariants[] items = RegisteredItemVariants.values();
        for (int i = 0; i < items.length; i++) {
            Bukkit.removeRecipe(new NamespacedKey(islandSurvivalCraft, items[i].getVariedItem().getItemName()));
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onHeld(PlayerItemHeldEvent e) {
        ItemStack item = e.getPlayer().getInventory().getItem(e.getNewSlot());
        ItemVariant variedItem = getVariedItemFromItemStack(item);
        if (variedItem != null) {
            variedItem.onItemHeld(item, e.getPlayer());
        } else {
            item = e.getPlayer().getInventory().getItem(e.getPreviousSlot());
            variedItem = getVariedItemFromItemStack(item);
            if (variedItem != null) {
                variedItem.onItemPutAway(item, e.getPlayer());
            }
        }
    }

    @EventHandler(priority = EventPriority.LOWEST)
    public void onInventoryInteract(InventoryClickEvent e) {
        ItemStack item = e.getCurrentItem();
        ItemVariant variedItem = getVariedItemFromItemStack(item);
        if (e.getSlotType() == SlotType.RESULT) {
            if (variedItem != null) {
                if (variedItem.onItemCrafted(item, e.getWhoClicked())) {
                    e.setCancelled(true);
                }            
            }
        } else if (e.getSlotType() == SlotType.QUICKBAR) {
            if (e.getWhoClicked().getInventory().getHeldItemSlot() == e.getSlot()) {
                if (variedItem != null) {
                    variedItem.onItemPutAway(item, e.getWhoClicked());
                } else if ((variedItem = getVariedItemFromItemStack(e.getCursor())) != null) {
                    variedItem.onItemHeld(item, e.getWhoClicked());
                }
            }
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onPlayerDropItem(PlayerDropItemEvent e) {
        ItemStack item = e.getItemDrop().getItemStack();
        ItemVariant variedItem = getVariedItemFromItemStack(item);
        if (variedItem != null) {
            variedItem.onItemPutAway(item, e.getPlayer());
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onEntityPickup(EntityPickupItemEvent e) {
        ItemStack item = e.getItem().getItemStack();
        if (e.getEntityType() == EntityType.PLAYER) {
            ItemVariant variedItem = getVariedItemFromItemStack(item);
            if (variedItem != null) {
                Player player = (Player) e.getEntity();
                if (player.getInventory().getHeldItemSlot() == player.getInventory().firstEmpty()) {
                    variedItem.onItemHeld(item, player);
                }
            }
        }
    }

    @EventHandler(priority = EventPriority.LOWEST)
    public void onPlayerInteraction(PlayerInteractEvent e) {
        if (e.useItemInHand() == Result.DENY || (e.getAction() != Action.RIGHT_CLICK_AIR && e.getAction() != Action.RIGHT_CLICK_BLOCK)) return;
        ItemStack item = e.getItem();
        ItemVariant variedItem = getVariedItemFromItemStack(item);
        if (variedItem != null) {
            if(variedItem.onItemRightClicked(item, e.getPlayer(), e.getHand(), e.getClickedBlock(), e.getBlockFace())) {
                e.setCancelled(true);
                e.setUseItemInHand(Result.DENY);
            }
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onPlayerJoin(PlayerJoinEvent e) {
        initializeInventoryVariedItems(e.getPlayer().getInventory());
        ItemStack item = e.getPlayer().getInventory().getItemInMainHand();
        ItemVariant variedItem = getVariedItemFromItemStack(item); //Check to see of we can find a variation of the item.
        if (variedItem != null) {
            variedItem.onItemHeld(item, e.getPlayer());
        }
    }

    private void initializeInventoryVariedItems(Inventory inv) {
        RegisteredItemVariants[] registeredVariedItems = RegisteredItemVariants.values();
        for (RegisteredItemVariants registeredVariedItem : registeredVariedItems) {
            ItemVariant variedItem = registeredVariedItem.getVariedItem();
            
            Material[] bases = variedItem.getBaseItems();
            for (Material base : bases) {
                HashMap<Integer, ? extends ItemStack> potentials = inv.all(base);
                for (Entry<Integer, ? extends ItemStack> entry : potentials.entrySet()) {
                    if (registeredVariedItem.getVariedItem().getRecipe() != null) {
                        NamespacedKey key = new NamespacedKey(islandSurvivalCraft, variedItem.getItemName() + ".identifier");
                        String identifier = entry.getValue().getItemMeta().getPersistentDataContainer().get(key, variedItem.getIdentifier());
                        if (identifier != null) {
                            variedItem.initializeInstanceOfItem(entry.getValue());
                        }
                    } else {
                        variedItem.initializeInstanceOfItem(entry.getValue());
                    }
                }
            }
        }
    }

    private ItemVariant getVariedItemFromItemStack(ItemStack item) {
        if (item == null || item.getType() == Material.AIR) return null;
        ItemMeta meta = item.getItemMeta();
        RegisteredItemVariants[] items = RegisteredItemVariants.values();
        for (int i = 0; i < items.length; i++) {
            ItemVariant current = items[i].getVariedItem();
            if (current.getRecipe() == null) {
                for (Material material : current.getBaseItems()) {
                    if (item.getType() == material) {
                        return current;
                    }
                }
            } 
            NamespacedKey key = new NamespacedKey(islandSurvivalCraft, current.getItemName());
            String identifier = meta.getPersistentDataContainer().get(key, current.getIdentifier());
            if (identifier != null) {
                if (current.getIdentifier().getID().equals(identifier)) {
                    return current;
                }
            }
        }

        return null;
    }
}