package net.reslate.islandsurvivalcraft.interaction.items.variations;

import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.entity.HumanEntity;
import org.bukkit.entity.Player;
import org.bukkit.inventory.EquipmentSlot;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.Recipe;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.inventory.meta.MapMeta;
import org.bukkit.map.MapCanvas;
import org.bukkit.map.MapPalette;
import org.bukkit.map.MapRenderer;
import org.bukkit.map.MapView;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;

import net.reslate.islandsurvivalcraft.IslandSurvivalCraft;
import net.reslate.islandsurvivalcraft.interaction.items.ItemVariant;
import net.reslate.islandsurvivalcraft.interaction.items.ItemVariantIdentifier;
import net.reslate.islandsurvivalcraft.utilities.GeneralUtilities;
import net.reslate.islandsurvivalcraft.utilities.datatypes.Point2;
import net.reslate.islandsurvivalcraft.world.WorldInfo;
import net.reslate.islandsurvivalcraft.world.Information.IslandInformation;
import net.reslate.islandsurvivalcraft.world.Information.IslandInformationManager;

import java.awt.Color;

public class IslandMapItem extends MapRenderer implements ItemVariant {
    IslandSurvivalCraft plugin;
    ItemVariantIdentifier identifier;
    Color mainColor = new Color(0, 180, 0);

    @SuppressWarnings("deprecation")
    @Override
    public void render(MapView map, MapCanvas canvas, Player player) {
        WorldInfo worldInfo = plugin.getWorldInfoManager().retrieve(map.getWorld().getName());
        IslandInformationManager islandInformationManager = worldInfo.getIslandInfoManager();
        int blocksPerPixel = (int) (1 * Math.pow(2, map.getScale().getValue()));
        for (int x = 0; x < 128; x += 1) {
            for (int y = x % 2; y < 128; y += 2) {
                if (canvas.getPixel(x, y) != MapPalette.matchColor(mainColor)) {
                    int actualX = blocksPerPixel * (x - Byte.MAX_VALUE/2) + map.getCenterX();
                    int actualZ = blocksPerPixel * (y - Byte.MAX_VALUE/2) + map.getCenterZ();
                    Point2 pCoords = new Point2(actualX, actualZ);
                    if (islandInformationManager.isChunkIslandInformationLoaded(GeneralUtilities.worldToChunkCoordinates(pCoords), false)) {
                        IslandInformation info = islandInformationManager.getIslandInformation(pCoords, false);
                        if (info != null) {
                            canvas.setPixel(x, y, MapPalette.matchColor(mainColor));
                        }
                    } else if (worldInfo.getIslandMap().isIsland(actualX, actualZ)) {
                        canvas.setPixel(x, y, MapPalette.matchColor(mainColor));
                    }
                }
            }
        }
    }

    @Override
    public void initialize(IslandSurvivalCraft islandSurvivalCraft) {
        this.plugin = islandSurvivalCraft;
    }

    @Override
    public Recipe getRecipe() {
        return null;
    }

    @Override
    public String getItemName() {
        return "Island_Map";
    }

    @Override
    public Material[] getBaseItems() {
        return new Material[] {Material.FILLED_MAP};
    }

    @Override
    public void assignIdentifier(ItemVariantIdentifier identifier) {
        this.identifier = identifier;
    }

    @Override
    public ItemVariantIdentifier getIdentifier() {
        return this.identifier;
    }

    @Override
    public boolean needInitialization() {
        return this.plugin == null;
    }

    @Override
    public boolean onItemCrafted(ItemStack item, HumanEntity crafter) {
        return false;
    }

    @Override
    public boolean onItemRightClicked(ItemStack item, HumanEntity clicker, EquipmentSlot hand, Block block,
            BlockFace blockFace) {
                if (((Player)clicker).isSneaking()) {
                    ItemMeta meta = item.getItemMeta();
                    PersistentDataContainer container = meta.getPersistentDataContainer();
                    NamespacedKey key = new NamespacedKey(plugin, getItemName() + ".state");
                    if (!container.has(key, PersistentDataType.BYTE)) {
                        container.set(key, PersistentDataType.BYTE, (byte) 0);
                    }
                    if (container.get(key, PersistentDataType.BYTE) == 0) {
                        container.set(key, PersistentDataType.BYTE, (byte) 1);
                        MapMeta mapMeta = (MapMeta) item.getItemMeta();
                        mapMeta.getMapView().addRenderer(this);
                    } else {
                        container.set(key, PersistentDataType.BYTE, (byte) 0);
                        MapMeta mapMeta = (MapMeta) item.getItemMeta();
                        mapMeta.getMapView().removeRenderer(this);
                    }

                    item.setItemMeta(meta);
                }
        return false;
    }

    @Override
    public void onItemHeld(ItemStack heldItem, HumanEntity holder) {
    }

    @Override
    public void onItemPutAway(ItemStack item, HumanEntity holder) {
    }

    @Override
    public void initializeInstanceOfItem(ItemStack item) {
        MapMeta mapMeta = (MapMeta) item.getItemMeta();
        PersistentDataContainer container = mapMeta.getPersistentDataContainer();
        NamespacedKey key = new NamespacedKey(plugin, getItemName() + ".state");

        if (container.has(key, PersistentDataType.BYTE)) {
            if (container.get(key, PersistentDataType.BYTE) == 1) {
                mapMeta = (MapMeta) item.getItemMeta();
                mapMeta.getMapView().addRenderer(this);
                item.setItemMeta(mapMeta);
            }
        }
    }
}
